import { csv, select } from 'd3';
import { scatterPlot } from './scatterPlot';

// importing datset
const csvUrl =
'https://gist.githubusercontent.com/BVNPK001/7304cfc7500f79c1e69068ca8a979d59/raw/8135a993e7a55a5e3e938cb1223276b0d74a4ef5/iris.csv'

// reading data
const parseRow = (d) => {
  d.sepal_length = +d.sepal_length;
  d.sepal_width = +d.sepal_width;
  d.petal_length = +d.petal_length;
  d.petal_width = +d.petal_width;
  return d;
};

const width = window.innerWidth;
const height = window.innerHeight;
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

const xValue = (d) => d.petal_length;
const yValue = (d) => d.petal_width;
const rxValue = (d) => d.sepal_length;
const ryValue = (d) => d.sepal_width;
const group = (d) => d.species;

// Creating the plot
const main = async () => {
  const data = await csv(csvUrl, parseRow);

  const x = xValue((d) => d.petal_width);

  const y = yValue((d) => d.sepal_length);

  // coloring the marks in the scatter plot
  const color = d3
    .scaleOrdinal()
    .domain(['setosa', 'versicolor', 'virginica'])
    .range(['red ', 'blue', 'green ']);

  const marks = data.map((d) => ({
    x: xValue((d) => d.petal_width),
    y: yValue((d) => d.sepal_length),
    color: color(group(d)),
    rx: rxValue(d),
    ry: ryValue(d),
  }));

  // plotting the scatter plot
  const plot = scatterPlot()
    .width(width)
    .height(height)
    .data(await csv(csvUrl, parseRow))
    .xValue((d) => d.petal_width)
    .yValue((d) => d.sepal_length)
    .margin({
      top: 20,
      right: 130,
      bottom: 80,
      left: 120,
    })
    .radius(5);

  // Title of the plot
  svg
    .append('text')
    .attr('x', 400)
    .attr('y', 15)
    .text('Iris Scatterplot')
    .style('font-size', '20px')
    .attr('alignment-baseline', 'middle');

  // Legends creation
  const legend = svg
    .selectAll('legend')
    .data(color.domain())
    .enter()
    .append('g')
    .attr('class', 'legend')
    .attr('transform', function (d, i) {
      return 'translate(0,' + i * 20 + ')';
    });

  // Labeling the Legend with color and shape
  legend
    .append('rect')
    .attr('x', 150)
    .attr('y', 30)
    .attr('width', 15)
    .attr('height', 15)
    .style('fill', color);

  //Labeling the Legends with names
  legend
    .append('text')
    .attr('x', 170)
    .attr('y', 35)
    .attr('dy', '.35em')
    .style('text-anchor', 'start')
    .style('font-style', 'italic')
    .text(function (d) {
      return d;
    });

  // Coloring the species
  svg
    .selectAll('circle')
    .data(marks)
    .join('circle')
    .attr('cx', (d) => d.x)
    .attr('cy', (d) => d.y)
    .attr('fill', (d) => d.color)
    .attr('rx', (d) => d.rx)
    .attr('ry', (d) => d.ry);

  //Labeling the x-axis
  svg
    .append('text')
    .attr('class', 'x label')
    .attr('text-anchor', 'end')
    .attr('x', innerWidth / 2)
    .attr('y', 480)
    .attr('stroke', 'red')
    .text('Petal Length');

  //Labeling the y-axis
  svg
    .append('text')
    .attr('class', 'y label')
    .attr('text-anchor', 'end')
    .attr('x', -innerHeight / 2)
    .attr('y', 0)
    .attr('dy', '3em')
    .attr('transform', 'rotate(-90)')
    .attr('stroke', 'blue')
    .text('Sepal Length');

  const columns = [
    'petal_width',
    'sepal_width',
    'petal_length',
    'sepal_length',
  ];
  let i = 0;
  setInterval(() => {
    plot.xValue(
      (d) => d[columns[i % columns.length]]
    );
    svg.call(plot);
    i++;
  }, 2000);
};
main();
